﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using C1.WPF;
using C1.WPF.C1Chart;
using C1.WPF.C1Chart.Extended;
using System.Windows.Controls.Primitives;
using System.Windows.Media.Animation;
using System.Reflection;
using System.IO;
using System.Globalization;

namespace ChartTrendlines
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        // ** fields
        double _min = double.NaN, _max = double.NaN;
        double _minRange = 7.0 / 365;
        
        public MainWindow()
        {
            InitializeComponent();
            Loaded += new RoutedEventHandler(MainWindow_Loaded);
            LoadData();
        }

        void MainWindow_Loaded(object sender, RoutedEventArgs e)
        {
            if (Foreground is SolidColorBrush)
            {
                Color clr = ((SolidColorBrush)Foreground).Color;
                slider.Box.Fill = new SolidColorBrush(Color.FromArgb(32, clr.R, clr.G, clr.B));
                slider.Box.Stroke = new SolidColorBrush(Color.FromArgb(200, clr.R, clr.G, clr.B));
            }
        }

        void LoadData()
        {
            CSVData data = new CSVData();

            using (Stream stream = Assembly.GetExecutingAssembly().
                GetManifestResourceStream("ChartTrendlines.Resources.GasPrices.csv"))
            {
                data.Read(stream, true);
            }

            int len = data.Length;
            GasPriceData[] gpdata = new GasPriceData[len];

            for (int i = 0; i < len; i++)
            {
                gpdata[i] = new GasPriceData(DateTime.Parse(data[i, 0], CultureInfo.InvariantCulture),
                    double.Parse(data[i, "Price"], CultureInfo.InvariantCulture));
            }   

            _min = gpdata[0].Date.ToOADate();
            _max = gpdata[len - 1].Date.ToOADate();

            if (len > 0)
            {
                chart.BeginUpdate();
                CreateDataSeries(chart, gpdata, false);
                chart.ChartType = ChartType.Line;
                chart.View.AxisX.AnnoFormat = "yyyy";
                chart.View.AxisX.Min = _min;
                chart.View.AxisX.Max = _max;
                chart.View.AxisY.AnnoFormat = "c2";
                chart.EndUpdate();

                chartZoom.BeginUpdate();
                CreateDataSeries(chartZoom, gpdata, true);
                chartZoom.ChartType = ChartType.Line;
                chartZoom.View.AxisX.AnnoFormat = "MMM-yyyy";
                chartZoom.View.AxisX.Max = _min + 0.25 * (_max - _min);
                chartZoom.View.AxisY.Title = "Gas Price (Dollars per gallon)";
                chartZoom.View.AxisY.AnnoFormat = "c2";
             
                chartZoom.EndUpdate();
            }

        }

        // ** utilities
        void CreateDataSeries(C1Chart chart, GasPriceData[] data, bool showTrend)
        {
            chart.Data.Children.Clear();


            int len = data.Length;
            DateTime[] days = new DateTime[len];
            double[] price = new double[len];
            
            for (int i = 0; i < len; i++)
            {
                days[i] = data[i].Date;
                price[i] = data[i].Price;
                
            }

            XYDataSeries ds = new XYDataSeries(); 
            ds.Label = "Regular";
            ds.XValuesSource = days;
            ds.ValuesSource = price;
            ds.ConnectionStroke = new SolidColorBrush(Colors.Blue);
            chart.Data.Children.Add(ds);

            //add trend line
            if (showTrend)
            {
                TrendLine tl = new TrendLine();
                tl.Label = "Trend";
                tl.XValuesSource = days;
                tl.ValuesSource = price;
                tl.ConnectionStroke = new SolidColorBrush(Colors.Red);
                chart.Data.Children.Add(tl);          
            }

            foreach (DataSeries ser in chart.Data.Children)
                ser.ConnectionStrokeThickness = 1;

            chart.View.AxisX.IsTime = true;
            chart.View.AxisX.AnnoPosition = AnnoPosition.Near;
        }

        // ** zooming
        double Minimum
        {
            get { return double.IsNaN(_min) ? chart.View.AxisX.Min : _min; }
            set
            {
                if (value != _min)
                {
                    _min = value;
                    Update();
                }
            }
        }

        double Maximum
        {
            get { return double.IsNaN(_max) ? chart.View.AxisX.Max : _max; }
            set
            {
                if (value != _max)
                {
                    _max = value;
                    Update();
                }
            }
        }

        void Update()
        {
            chartZoom.BeginUpdate();
            chartZoom.View.AxisX.Min = Minimum;
            chartZoom.View.AxisX.Max = Maximum;
            chartZoom.EndUpdate();
        }

        // ** overrides
        protected override Size MeasureOverride(Size availableSize)
        {
            // adjust margin if necessary
            {
                Thickness mars = chart.View.Margin;
                double w = slider.ThumbWidth;
                mars.Left -= w - chart.Padding.Left; mars.Right -= w - chart.Padding.Right;
                mars.Top += chart.Padding.Top; mars.Bottom += chart.Padding.Bottom;
                slider.Margin = mars;
                //slc.Margin = mars;
            }

            return base.MeasureOverride(availableSize);
        }

        // ** event handlers
        void slider_LowerValueChanged(object sender, EventArgs e)
        {
            if (chart != null && chartZoom != null)
            {
                if (slider.UpperValue - slider.LowerValue < _minRange)
                    slider.LowerValue = slider.UpperValue - _minRange;

                Axis ax = chart.View.AxisX;
                if (ax.Max != ax.Min)
                {
                    double min = ax.Min + slider.LowerValue * (ax.Max - ax.Min);
                    Minimum = min;
                }
            }
        }
        void slider_UpperValueChanged(object sender, EventArgs e)
        {
            if (chart != null && chartZoom != null)
            {
                if (slider.UpperValue - slider.LowerValue < _minRange)
                    slider.UpperValue = slider.LowerValue + _minRange;

                Axis ax = chart.View.AxisX;
                if (ax.Max != ax.Min)
                {
                    double max = ax.Min + slider.UpperValue * (ax.Max - ax.Min);
                    Maximum = max;
                }
            }
        }
    }


    public class ChartSlider : C1RangeSlider
    {
        // ** fields
        Rectangle _box;
        double _tw;

        // ** ctor
        public ChartSlider()
        {
            _box = new Rectangle();
            _box.Fill = new SolidColorBrush(Color.FromArgb(32, 255, 255, 255));
            _box.Stroke = new SolidColorBrush(Color.FromArgb(200, 255, 255, 255));
            _box.StrokeThickness = 1;
            _box.IsHitTestVisible = false;
            Grid.SetColumn(_box, 1);
            Grid.SetColumnSpan(_box, 3);
        }

        // ** object model
        public double ThumbWidth
        {
            get { return _tw; }
        }

        public Rectangle Box
        {
            get { return _box; }
        }

        // ** overrides
        public override void OnApplyTemplate()
        {
            base.OnApplyTemplate();

            // insert rectangle that represents zoom area
            Grid root = GetTemplateChild("HorizontalTemplate") as Grid;
            if (root != null)
                root.Children.Insert(0, _box);

            Thumb lthumb = GetTemplateChild("LowerThumb") as Thumb;
            if (lthumb != null)
            {
                _tw = lthumb.Width;
                if (double.IsNaN(_tw))
                {
                    lthumb.Measure(new Size(1000, 1000));
                    _tw = lthumb.DesiredSize.Width;
                }
                double w = Math.Ceiling(_tw);
                // adjust zoom area margins
                _box.Margin = new Thickness(w, 0, w, 0);

                // move thumb on the border of zoom area
                lthumb.RenderTransform = new TranslateTransform() { X = 0.5 * _tw };

                // move thumb on the top of slider visual tree
                root.Children.Remove(lthumb);
                root.Children.Add(lthumb);
            }

            Thumb uthumb = GetTemplateChild("UpperThumb") as Thumb;
            if (uthumb != null)
                // move thumb on the border of zoom area
                uthumb.RenderTransform = new TranslateTransform() { X = -0.5 * _tw };
 
            // stretch middle thumb vertically
            Thumb mthumb = GetTemplateChild("MiddleThumb") as Thumb;
            if (mthumb != null)
                mthumb.Height = double.NaN;
        }
    }
}
